/**
  *  \file client/dlg-build.h
  */
#ifndef PCC_V2_CLIENT_DLG_BUILD_H
#define PCC_V2_CLIENT_DLG_BUILD_H

#include <vector>
#include "client/wcost.h"
#include "client/winfo.h"
#include "cpluslib/holder.h"
#include "game/action.h"
#include "game/spec.h"
#include "ui/listbox.h"
#include "ui/ui.h"
#include "ui/window.h"
#include "ui/widgets.h"

class GPlanet;

class WComponentListbox : public UIListbox {
 protected:
    GPlanet*   tick_planet;
 public:
    WComponentListbox(int id, int lines, int width, GPlanet* planet);

 protected:
    void drawComponent(GfxCanvas& can, int line, const GComponent& com, TTechLevel tech_area, int have);
};

class WHullListbox : public WComponentListbox {
    std::vector<int>  content;
 public:
    WHullListbox(int id, int lines, int width, GPlanet* planet);
    ~WHullListbox();

    void drawPart(GfxCanvas& can, int from, int to);

    int   getCurrentHull() const;
    void  addHullId(int n);
    void  swapContents(std::vector<int>& new_content);
    void  scrollToHull(int n);
};

class WEngineListbox : public WComponentListbox {
 public:
    WEngineListbox(int id, int width, GPlanet* tick_planet);
    void drawPart(GfxCanvas& can, int from, int to);
};

class WBeamListbox : public WComponentListbox {
 public:
    WBeamListbox(int id, int width, GPlanet* tick_planet);
    void drawPart(GfxCanvas& can, int from, int to);
};

class WLauncherListbox : public WComponentListbox {
 public:
    WLauncherListbox(int id, int width, GPlanet* tick_planet);
    void drawPart(GfxCanvas& can, int from, int to);
};

class WOrderDisplay : public UIWidget {
    TBuildOrder o;
 public:
    WOrderDisplay();
    void draw(GfxCanvas& can, UpdateInfo& ui);
    void getLayoutInfo(LayoutInfo& info);
    void setBuildOrder(const TBuildOrder& o);
};

class WShipBuildScreen : public UIWindow {
    WHullListbox      hull_list;
    WEngineListbox    engine_list;
    WBeamListbox      beam_list;
    WLauncherListbox  torp_list;

    WHullInfo         hull_info;
    WBeamInfo         beam_info;
    WTorpInfo         torp_info;
    WEngineInfo       engine_info;

 protected:
    GPlanetTransfer           transfer;
    GStarbaseBuildShipAction  action;

 private:
    WItemCostDisplay      item_cost_dpy;
    WOrderDisplay         order_dpy;

    UICardGroup       cards;
    UICardGroupTab    tabs;

    // Must be declared exactly here so that it dies first.
 protected:
    ObjectHolder      h;

    void updateHullPage();
    void updateBeamPage();
    void updateEnginePage();
    void updateLauncherPage();
    void updateBuildOrder();
 public:
    WShipBuildScreen(string_t name, GPlanet& the_planet);
    ~WShipBuildScreen();
};

class WShipBuildScreenWithActor : public WShipBuildScreen {
    void onOk();
    void onCancel();
    void onCancelBuild();
 public:
    WShipBuildScreenWithActor(GPlanet& the_planet);
    ~WShipBuildScreenWithActor();
};

#endif
