/**
  *  \file client/scr-planet.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "client/actions.h"
#include "client/cscreen.h"
#include "client/dlg-list.h"
#include "client/marks.h"
#include "client/screens.h"
#include "cpluslib/format.h"
#include "game/global.h"
#include "game/objlist.h"
#include "game/planet.h"
#include "game/ship.h"
#include "gfx/font.h"
#include "gfx/gfx.h"
#include "gfx/palette.h"
#include "ui/msgbox.h"
#include "ui/uiglobal.h"
#include "util/misc.h"
#include "util/nls.h"
#include "client/dlg-tech.h"
#include "ui/layout.h"
#include "client/dlg-structure.h"
#include "client/dlg-tax.h"
#include "ui/table.h"
#include "client/dlg-supplies.h"
#include "game/action.h"

namespace {
    class WPlanetSelect : public WObjectSelectionTile {
     public:
        WPlanetSelect();
        void drawData(GfxCanvas& can);
    };

    class WPlanetScreen : public WControlScreen {
        void* sig_handle;
     public:
        WPlanetScreen();
        ~WPlanetScreen();
        void update();
        bool handleEvent(const UIEvent& e, bool second_pass);
    };

    class WPlanetEconomyTile : public WTile {
     public:
        WPlanetEconomyTile();
        void drawFixed(GfxCanvas& can);
        void drawData(GfxCanvas& can);
        void addControls(UIGroup& g, ObjectHolder& h);
    };

    class WPlanetColonistTile : public WTile {
     public:
        WPlanetColonistTile();
        void drawFixed(GfxCanvas& can);
        void drawData(GfxCanvas& can);
        void addControls(UIGroup& g, ObjectHolder& h);
    };

    class WPlanetNativeTile : public WTile {
     public:
        WPlanetNativeTile();
        void drawFixed(GfxCanvas& can);
        void drawData(GfxCanvas& can);
    };
}

/***************************** WPlanetSelect *****************************/

WPlanetSelect::WPlanetSelect()
    : WObjectSelectionTile(getDisplayedTurn().getPlanetType())
{ }

void
WPlanetSelect::drawData(GfxCanvas& can)
{
    GPlanet& pp = getDisplayedTurn().getPlanet(getCurrent());

    // FIXME: this is very incomplet
    int y = 0, rem = 0;
    if (getType().isMarked(getCurrent())) {
        drawBackground(can, GfxRect(0, 0, 15, font_heights[FONT_NORMAL]));
        getSkin().getParameters().color = standard_colors[COLOR_GREENBLACK];
        drawSelection(can, getSkin(), extent.x + 5, extent.y + 7, 1, 2);
        rem = 15;
    }
    setColor(UIColor::tc_Static);
    outTextF(can, FONT_NORMAL, rem, y, extent.w - rem, pp.getName());
    y += font_heights[FONT_NORMAL]+2;

    /* location, climate */
    outTextF(can, FONT_NORMAL, 0, y, 130, format(_("Climate: %s"), pp.getTempStr()));
    if (int sector = pp.getPos().getSectorNumber())
        outTextF(can, FONT_NORMAL, 130, y, extent.w - 130,
                 format(_("Location: (%d,%d) - Sector %d"),
                        pp.getPos().x, pp.getPos().y, sector));
    else
        outTextF(can, FONT_NORMAL, 130, y, extent.w - 130,
                 format(_("Location: (%d,%d)"),
                        pp.getPos().x, pp.getPos().y));
    y += font_heights[FONT_NORMAL]+2;

    // FIXME: I don't really like this because it uses hard
    // measurements (the same as in PCC 1.x)
    /*  |-35-|---70---|5|---95---|30|----175--------|
        Col:  N,NNN,NNN Race      xx% -> happy (100)
        Nat:  N,NNN,NNN Race
        |0   |35      |105|110   |205|235  */
    
    /* colonists */
    outTextF(can, FONT_NORMAL, 0,   y, 35, _("Col:"));
    if (pp.getOwner()) {
        outTextF(can, FONT_NORMAL, 110, y, 95, getAdjName(pp.getOwner()));
        outTextF(can, FONT_NORMAL, 235, y, 155,
                 format(" -> %s (%d)", pp.getHappyStr(pp.getColonistHappiness()),
                        pp.getColonistHappiness()));
        getSkin().getParameters().just_x = 2;
        outTextF(can, FONT_NORMAL, 105, y, 70, clansToString(pp.getColonists()));
        outTextF(can, FONT_NORMAL, 235, y, 30, format("%d%%", pp.getColonistTax()));
        getSkin().getParameters().just_x = 0;
    } else {
        outTextF(can, FONT_NORMAL, 35, y, 70+5+95+30+155, _("none"));
    }
    y += font_heights[FONT_NORMAL];

    /* natives */
    outTextF(can, FONT_NORMAL, 0, y, 35, _("Nat:"));
    if (pp.getNativeRace()) {
        outTextF(can, FONT_NORMAL, 110, y, 95, pp.getNativeRaceName(pp.getNativeRace()));
        outTextF(can, FONT_NORMAL, 235, y, 155,
                 format(" -> %s (%d)", pp.getHappyStr(pp.getNativeHappiness()),
                        pp.getNativeHappiness()));
        getSkin().getParameters().just_x = 2;
        outTextF(can, FONT_NORMAL, 105, y, 70, clansToString(pp.getNatives()));
        outTextF(can, FONT_NORMAL, 235, y, 30, format("%d%%", pp.getNativeTax()));
        getSkin().getParameters().just_x = 0;
    } else {
        outTextF(can, FONT_NORMAL, 35, y, 70+5+95+30+155, _("none"));
    }
    y += font_heights[FONT_NORMAL]+2;

    /* industry */
    outTextF(can, FONT_NORMAL, 0, y, extent.w,
             format(_("Mines: %d; Factories: %d; Defense Posts: %d"),
                    pp.getStructures(str_Mines),
                    pp.getStructures(str_Factories),
                    pp.getStructures(str_Defense)));
    y += font_heights[FONT_NORMAL];

    outTextF(can, FONT_NORMAL, 0, y, extent.w,
             format(_("%d mc; %d kt supplies"),
                    pp.getCargo(el_Money), pp.getCargo(el_Supplies)));
    y += font_heights[FONT_NORMAL];

    /* minerals */
    outTextF(can, FONT_NORMAL, 0, y, 50, _("Mined:"));
    getSkin().getParameters().just_x = 2;
    outTextF(can, FONT_NORMAL, 130, y, 80, format("%dN", pp.getCargo(el_Neutronium)));
    outTextF(can, FONT_NORMAL, 210, y, 80, format("%dT", pp.getCargo(el_Tritanium)));
    outTextF(can, FONT_NORMAL, 290, y, 80, format("%dD", pp.getCargo(el_Duranium)));
    outTextF(can, FONT_NORMAL, 370, y, 80, format("%dM", pp.getCargo(el_Molybdenum)));
    y += font_heights[FONT_NORMAL];
    
    outTextF(can, FONT_NORMAL, 130, y, 80, format("%dN", pp.getOreGround(el_Neutronium)));
    outTextF(can, FONT_NORMAL, 210, y, 80, format("%dT", pp.getOreGround(el_Tritanium)));
    outTextF(can, FONT_NORMAL, 290, y, 80, format("%dD", pp.getOreGround(el_Duranium)));
    outTextF(can, FONT_NORMAL, 370, y, 80, format("%dM", pp.getOreGround(el_Molybdenum)));
    getSkin().getParameters().just_x = 0;
    outTextF(can, FONT_NORMAL, 0, y, 50, _("Ground:"));
    y += font_heights[FONT_NORMAL] + 2;

    outTextF(can, FONT_NORMAL, 0, y, extent.w,
             format(_("Friendly Code: %s"), pp.getFCode()));
}

int
choosePlanet()
{
    GGameTurn& trn = getDisplayedTurn();
    int nplanets = trn.getPlanetType().getCount();
    if (nplanets == 0) {
        messageBox(_("You do not have any planets. "
                     "Unload colonists from a starship to a planet to colonize it."),
                   _("Planets"));
        return 0;
    } else if (nplanets == 1) {
        return trn.getPlanetType().getCurrent();
    } else {
        WPlanetSelect selector;
        return doSelectWindow(selector, _("Planet"));
    }
}

/*************************** WPlanetEconomyTile **************************/

WPlanetEconomyTile::WPlanetEconomyTile()
    : WTile(GfxPoint(315, font_heights[FONT_NORMAL] * 8))
{ }

static void
drawPlanetEconomyTile(UIAbstractTableWriter& w, bool all_cols)
{
    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Static, _("Mineral Mines:"));
    w.writeFixedCell(UIColor::tc_Static, _("Factories:"));
    w.writeFixedCell(UIColor::tc_Static, _("Defense Posts:"));
    w.writeFixedCell(UIColor::tc_Static, _("Neutronium:"));
    w.writeFixedCell(UIColor::tc_Static, _("Tritanium:"));
    w.writeFixedCell(UIColor::tc_Static, _("Duranium:"));
    w.writeFixedCell(UIColor::tc_Static, _("Molybdenum:"));
    w.writeFixedCell(UIColor::tc_Static, _("Supplies:"));
    if (!all_cols)
        return;

    const GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();

    w.newRColumn(6*8);
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getStructures(str_Mines)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getStructures(str_Factories)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getStructures(str_Defense)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getCargo(el_Neutronium)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getCargo(el_Tritanium)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getCargo(el_Duranium)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getCargo(el_Molybdenum)));
    w.writeVariableCell(UIColor::tc_Green, itoa(p.getCargo(el_Supplies)));

    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Green, "");
    w.writeFixedCell(UIColor::tc_Green, "");
    w.writeFixedCell(UIColor::tc_Green, "");
    w.writeFixedCell(UIColor::tc_Green, "kt");
    w.writeFixedCell(UIColor::tc_Green, "kt");
    w.writeFixedCell(UIColor::tc_Green, "kt");
    w.writeFixedCell(UIColor::tc_Green, "kt");
    w.writeFixedCell(UIColor::tc_Green, "kt");

    w.finish();
}

void
WPlanetEconomyTile::drawFixed(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 0), FONT_NORMAL);
    drawPlanetEconomyTile(tw, false);
}

void
WPlanetEconomyTile::drawData(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 0), FONT_NORMAL);
    drawPlanetEconomyTile(tw, true);
}

void
WPlanetEconomyTile::addControls(UIGroup& g, ObjectHolder& h)
{
    GfxRect r = getInnerSize(0, 0);

    addControl(g, GfxRect(r.w-20,  0, 20, 20), h.add(new UIButton("G", 'g')));
    addControl(g, GfxRect(r.w-45, 30, 20, 20), h.add(new UIButton("B", 'b')));
    addControl(g, GfxRect(r.w-20, 30, 20, 20), h.add(new UIButton("M", 'm')));
    addControl(g, GfxRect(r.w-45, 55, 20, 20), h.add(new UIButton("S", 's')));
    addControl(g, GfxRect(r.w-20, 55, 20, 20), h.add(new UIButton("D", 'd')));
    addControl(g, GfxRect(r.w-20, 90, 20, 20), h.add(new UIButton("C", 'c')));
}

/************************** WPlanetColonistTile **************************/

WPlanetColonistTile::WPlanetColonistTile()
    : WTile(315, 3 * font_heights[FONT_NORMAL])
{ }

static void
drawPlanetColonistTile(UIAbstractTableWriter& w, bool all_cols)
{
    w.newLColumn();
    w.writeFixedCell(UIColor::tc_Static, _("Population:"));
    w.writeFixedCell(UIColor::tc_Static, _("Tax Rate:"));
    w.writeFixedCell(UIColor::tc_Static, _("Colonists are"));

    const GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();

    w.newLColumn(200);
    w.writeVariableCell(UIColor::tc_Green, clansToString(p.getColonists()));
    w.writeVariableCell(UIColor::tc_Green, format("%d%%", p.getColonistTax()));

    int happy = p.getColonistHappiness();
    UISkin::Color c = (happy <= 40
                       ? UIColor::tc_Red
                       : happy < 70
                       ? UIColor::tc_Yellow
                       : UIColor::tc_Green);
    w.writeVariableCell(c, format("%s (%d)", p.getHappyStr(happy), happy));

    w.finish();
}

void
WPlanetColonistTile::drawFixed(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 20), FONT_NORMAL);
    drawPlanetColonistTile(tw, false);
}

void
WPlanetColonistTile::drawData(GfxCanvas& can)
{
    UITableWriter tw(can, getSkin(), getInnerSize(5, 20), FONT_NORMAL);
    drawPlanetColonistTile(tw, true);
}

void
WPlanetColonistTile::addControls(UIGroup& g, ObjectHolder& h)
{
    GfxRect r = getInnerSize(0, 0);
    addControl(g, GfxRect(r.w - 20, r.h - 20, 20, 20), h.add(new UIButton("T", 't')), true);
}

/*************************** WPlanetNativeTile ***************************/

WPlanetNativeTile::WPlanetNativeTile()
    : WTile(315, 4 * font_heights[FONT_NORMAL])
{ }

void
WPlanetNativeTile::drawFixed(GfxCanvas& /*can*/)
{ }

void
WPlanetNativeTile::drawData(GfxCanvas& can)
{
    const GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();
    const int y = font_heights[FONT_NORMAL];

    setColor(UIColor::tc_Static);

    if (!p.getNativeRace()) {
        // FIXME: NLS might need more space
        outTextF(can, FONT_NORMAL, 5, 0, extent.w-5, _("There are no natives on this planet."));
        drawBackground(can, GfxRect(5, y, extent.w-5, 3*y));
    } else {
        // This erases the sentence from above because the table
        // writer doesn't (can't?) do that
        drawBackground(can, GfxRect(5, 0, extent.w-5, y));

        UITableWriter tw(can, getSkin(), getInnerSize(5, 0), FONT_NORMAL);

        tw.newLColumn();
        tw.writeFixedCell(UIColor::tc_Static, _("Race:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Population:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Tax Rate:"));
        tw.writeFixedCell(UIColor::tc_Static, _("Natives are"));

        tw.newLColumn(200);
        tw.writeVariableCell(UIColor::tc_Green, string_t(p.getNativeRaceName(p.getNativeRace())) + ", " + p.getNativeGovernmentName(p.getNativeGovernment()));
        tw.writeVariableCell(UIColor::tc_Green, clansToString(p.getNatives()));
        tw.writeVariableCell(UIColor::tc_Green, format("%d%%", p.getNativeTax()));

        int happy = p.getNativeHappiness();
        UISkin::Color c = (happy <= 40
                       ? UIColor::tc_Red
                       : happy < 70
                       ? UIColor::tc_Yellow
                       : UIColor::tc_Green);
        tw.writeVariableCell(c, format("%s (%d)", p.getHappyStr(happy), happy));

        tw.finish();
    }
}

/**************************** WPlanetFCodeTile ***************************/

WPlanetFCodeTile::WPlanetFCodeTile(GObjectType& type)
    : WTile(315, 4*font_heights[FONT_NORMAL]), type(type)
{ }

void
WPlanetFCodeTile::drawFixed(GfxCanvas& can)
{
    const int y = font_heights[FONT_NORMAL];

    setColor(UIColor::tc_Static);
    outText(can, FONT_NORMAL, 5, 0, _("Funds:"));
    outText(can, FONT_NORMAL, 5, y, _("Friendly Code:"));
}

void
WPlanetFCodeTile::drawData(GfxCanvas& can)
{
    const int y = font_heights[FONT_NORMAL];
    GPlanet& p = getDisplayedTurn().getPlanet(type.getCurrent());

    setColor(UIColor::tc_Green);
    outTextF(can, FONT_NORMAL, 65, 0, 170, format("%d mc", p.getCargo(el_Money)));
    outTextF(can, FONT_NORMAL, 125, y, 70, p.getFCode());

    string_t comment = "FIXME"; // FIXME
    if (comment.length()) {
        setColor(UIColor::tc_Yellow);
        outTextF(can, FONT_NORMAL, 5, 5*y/2, extent.w-35, comment);
    } else {
        setColor(UIColor::tc_Static);
        outTextF(can, FONT_NORMAL, 5, 5*y/2, extent.w-35, _("(press [F9] to add a note)"));
    }
}

void
WPlanetFCodeTile::addControls(UIGroup& g, ObjectHolder& h)
{
    GfxRect r = getInnerSize(0, 0);
    addControl(g, GfxRect(r.w-20, 0, 20, 20),      h.add(new UIButton("F", 'f')));
    addControl(g, GfxRect(r.w-30, r.h-20, 30, 20), h.add(new UIButton("F9", SDLK_F9)));
}

/**************************** WPlanetBaseTile ****************************/

WPlanetBaseTile::WPlanetBaseTile(bool on_base)
    : WTile(315, font_heights[FONT_TITLE]*2 + 2),
      on_base(on_base)
{ }

void
WPlanetBaseTile::drawFixed(GfxCanvas& can)
{ }

void
WPlanetBaseTile::drawData(GfxCanvas& can)
{
    GPlanet& p = getDisplayedTurn().getPlanetType().getPlanet();
    const int w = getExtent().w - 35;
    const char* s;

    setColor(UIColor::tc_White);
    getSkin().getParameters().just_x = 2;
    outTextF(can, FONT_TITLE, w, 0, w, _("Planet overview"));

    if (on_base) {
        s = _("Go to planet");
    } else if (p.hasBase()) {
        s = _("Go to starbase");
    } else if (p.isBuildingBase()) {
        setColor(UIColor::tc_Green);
        s = _("Starbase being built...");
    } else {
        s = _("Build starbase");
    }

    outTextF(can, FONT_TITLE, w, font_heights[FONT_TITLE]+2, w, s);
}

void
WPlanetBaseTile::addControls(UIGroup& g, ObjectHolder& h)
{
    const int w = getExtent().w - 30;
    const int he = font_heights[FONT_TITLE];
    addControl(g, GfxRect(w, 0, 30, he),
               h.add(new UIButton("F5", SDLK_F5)));
    addControl(g, GfxRect(w, he+2, 30, he),
               h.add(new UIButton("F8", SDLK_F8)));
}

/***************************** WPlanetScreen *****************************/

WPlanetScreen::WPlanetScreen()
    : WControlScreen(getDisplayedTurn().getPlanetType(), 2, GPlanet::ps_DirtyHead)
{
    addTile(holder.add(new WControlScreenHeaderTile(*this, WControlScreenHeaderTile::bAuto + WControlScreenHeaderTile::bAdd + WControlScreenHeaderTile::bSend,
                                                    GPlanet::ps_DirtyHead)));
    addTile(holder.add(new WPlanetEconomyTile()),
            _("Economy:"));
    addTile(holder.add(new WPlanetNativeTile()),
            _("Natives:"));
    addTile(holder.add(new WPlanetColonistTile()),
            _("Colonists:"));
    addTile(holder.add(new WPlanetFCodeTile(getDisplayedTurn().getPlanetType())));
    addTile(holder.add(new WPlanetBaseTile(false)));
    addChart(holder.add(new WScannerChartWidget()));

    sig_handle = getDisplayedTurn().sig_planet_changed.add(static_cast<WControlScreen*>(this),
                                                           &WControlScreen::updateId);
}

WPlanetScreen::~WPlanetScreen()
{
    getDisplayedTurn().sig_planet_changed.remove(sig_handle);
}

void
WPlanetScreen::update()
{
    int pid = getDisplayedTurn().getPlanetType().getCurrent();
    const GPlanet& planet = getDisplayedTurn().getPlanet(pid);
    setData(mkResId(rt_Climate, planet.getTemp()),
            planet.getPos(),
            planet.getName(),
            format(_("(Id #%d, %s - %dF)"),
                   pid, GPlanet::getTempStr(planet.getTemp()), planet.getTemp()));
}

static void
doPlanetXchg(WPlanetScreen* scr)
{
    WScanner* scanner = &scr->getScanner();
    int pid = scanner->getScannedPlanetId(scanner->getScannedShipId() != 0);
    if (pid && pid != scr->getType().getCurrent() && scr->getType().isExistant(pid))
        putCommand(cm_CS_GotoCS, 2, pid);
    else
        putKey(ss_Ctrl + SDLK_F1);
}

bool
WPlanetScreen::handleEvent(const UIEvent& e, bool second_pass)
{
    if (e.type == ev_Key) {
        GPlanet& planet = getDisplayedTurn().getPlanetType().getPlanet();
        switch(e.key.code) {
         case 'x':
            doPlanetXchg(this);
            return true;
         case 'c':
         case 'u':
            doPlanetTransfer(planet, e.key.code == 'u');
            return true;
         case 'f':
            doPlanetFCodeChange(planet);
            return true;
         case 'b':
         case SDLK_TAB:
            doPlanetBuildScreen(planet, 0);
            return true;
         case 'm':
            doPlanetBuildScreen(planet, 1);
            return true;
         case 'd':
            doPlanetBuildScreen(planet, 2);
            return true;
         case 't':
            doTaxation(planet);
            return true;
         case 's':
            WSellSuppliesDialog::doDialog(planet);
            return true;
         case SDLK_F8:
            if (planet.hasBase()) {
                putCommand(cm_CS_GotoCS, 3, planet.getId());
            } else {
                GPlanetTransfer bill(planet);
                GPlanetBuildStarbaseAction action(planet, bill, !planet.isBuildingBase());

                if (GError e = action.getError()) {
                    messageBox(e.getUserError(), _("Build Starbase"));
                    action.cancel();
                    bill.cancel();
                    return true;
                }

                if (planet.isBuildingBase()) {
                    if (UIMessageBox(_("You wanted to build a starbase at this planet. "
                                       "Cancel this order?"),
                                     _("Build Starbase")).doYesNoDialog()) {
                        action.commit();
                        bill.commit();
                    } else {
                        action.cancel();
                        bill.cancel();
                    }
                    getDisplayedTurn().doUpdates();
                } else {
                    static const UIKeyTranslator::Init xl[] = {
                        { SDLK_RETURN, cm_Enter },
                        { SDLK_ESCAPE, cm_Escape }
                    };

                    UIWindow win(_("Build Starbase"), UIWindow::BLUE_WINDOW, COLOR_GRAY);
                    win.setLayout(UIVBoxLayout::instance5);
                    ObjectHolder h;
                    win.add(h.add(new WCostDisplay(action,
                                                   (1 << el_Tritanium)
                                                   | (1 << el_Duranium)
                                                   | (1 << el_Molybdenum)
                                                   | (1 << el_Money))));
                    UIGroup bg(UIHBoxLayout::instance5);
                    bg.add(h.add(new UISpacer()));
                    if (action.isValid()) {
                        UIKeyString y(_("Yes")), n(_("No"));
                        win.add(h.add(new UIStaticText(_("Build this starbase?"), UIStaticText::BlackTitle)));
                        bg.add(h.add(new UIButton(cm_Enter,  y.getString(), y.getKey(), UIButton::aStop)));
                        bg.add(h.add(new UIButton(cm_Escape, n.getString(), n.getKey(), UIButton::aStop)));
                    } else {
                        win.add(h.add(new UIStaticText(_("You can't build a starbase here."), UIStaticText::BlackTitle)));
                        bg.add(h.add(new UIButton(cm_Escape, _("OK"), SDLK_RETURN, UIButton::aStop)));
                    }
                    bg.add(h.add(new UISpacer()));
                    win.add(bg);
                    win.add(h.add(new UIKeyTranslator(xl, countof(xl))));
                    win.pack();

                    ui_root->centerWidget(win);
                    ui_root->add(win);
                    int rv = ui_root->run();
                    ui_root->close(win);
                    if (rv == cm_Enter && action.isValid()) {
                        action.commit();
                        bill.commit();
                    } else {
                        action.cancel();
                        bill.cancel();
                    }
                    getDisplayedTurn().doUpdates();
                }
            }
            return true;

            /* planet screen keys:
               f9       comment
               ins      sim
               i        mail
               alt-r    reset
               c-s      undo supply sale
               g        give */
        }
    }
    return WControlScreen::handleEvent(e, second_pass);
}

int
doPlanetScreen()
{
    return WPlanetScreen().run();
}
