/**
  *  \file game/act-ammo.cc
  */

#include "game/action.h"
#include "cpluslib/assert.h"
#include "game/spec.h"
#include "cpluslib/misc.h"

enum {
    Fighter_TriCost   = 3,
    Fighter_MolCost   = 2,
    Fighter_MoneyCost = 100
};

GStarbaseBuildAmmoAction::GStarbaseBuildAmmoAction(GPlanetTransfer& pay, GCargoContainer& receiver)
    : GCargoCostTransaction(pay), pay(pay), receiver(receiver)
{
    for (unsigned i = 0; i < countof(added); ++i)
        added[i] = 0;
}

GStarbaseBuildAmmoAction::~GStarbaseBuildAmmoAction()
{ }

GCargoType
GStarbaseBuildAmmoAction::getCargoTypeFromIndex(int index)
{
    if (index > 0 && index <= NUM_TORPS)
        return (GCargoType(el_Torps + index-1));
    else if (index == NUM_TORPS+1)
        return el_Fighters;
    else
        ASSERT(0);
}

bool
GStarbaseBuildAmmoAction::canHave(int index) const
{
    return receiver.canHave(getCargoTypeFromIndex(index));
}

int32
GStarbaseBuildAmmoAction::getAmount(int index) const
{
    return receiver.getCurrent(getCargoTypeFromIndex(index));
}

void
GStarbaseBuildAmmoAction::setAmount(int index, int32 amount)
{
    // difference to total amount
    int32 diff = amount - getAmount(index);

    // we can't take away more than we have
    if (diff < 0 && -diff > added[index-1])
        diff = -added[index-1];

    // virtually add it to underlying object
    int32 done = receiver.addCargoEx(getCargoTypeFromIndex(index), diff, true);
    added[index-1] += done;

    // re-compute cost
    int32 t = 0, d = 0, m = 0, mc = 0;
    for (int i = 1; i <= NUM_TORPS; ++i) {
        if (added[i-1]) {
            t += added[i-1];
            d += added[i-1];
            m += added[i-1];
            mc += getTorp(i).getTorpCost() * added[i-1];
        }
    }
    if (added[NUM_TORPS]) {
        t += Fighter_TriCost * added[NUM_TORPS];
        m += Fighter_MolCost * added[NUM_TORPS];
        mc += Fighter_MoneyCost * added[NUM_TORPS];
    }

    setCost(el_Tritanium, t);
    setCost(el_Molybdenum, m);
    setCost(el_Duranium, d);
    setCost(el_Money, mc);
}

void
GStarbaseBuildAmmoAction::commit()
{
    GCargoCostTransaction::commit();
    receiver.mustCommit();
}

void
GStarbaseBuildAmmoAction::cancel()
{
    GCargoCostTransaction::cancel();
    receiver.mustCancel();
}

/** Get required tech upgrade. This returns zero when the specified
    thing can be immediately built, otherwise it returns the tech
    level needed to build. */
int
GStarbaseBuildAmmoAction::getTechUpgradeNeeded(int index) const
{
    if (index > 0 && index <= NUM_TORPS) {
        int tech = getTorp(index).getTech();
        if (tech > pay.getPlanet().getBaseTech(tl_Torpedo))
            return tech;
    }
    return 0;
}
