/**
  *  \file game/act-supplies.cc
  */

#include "game/action.h"
#include "cpluslib/assert.h"

/*********************** GPlanetSellSuppliesAction ***********************/

GPlanetSellSuppliesAction::GPlanetSellSuppliesAction(GPlanet& the_planet)
    : the_planet(the_planet), transfer(the_planet)
{
    // reserve everything for us
    reserved_sup = transfer.removeCargo(el_Supplies,
                                        transfer.getCurrent(el_Supplies),
                                        true);
    amount = 0;
}

GPlanetSellSuppliesAction::~GPlanetSellSuppliesAction()
{
    ASSERT(reserved_sup == 0);
}

int32
GPlanetSellSuppliesAction::getMinimum() const
{
    return 0;    // FIXME: should be modified later to allow for undo-supply-sale
}

int32
GPlanetSellSuppliesAction::getMaximum() const
{
    return reserved_sup;
}

void
GPlanetSellSuppliesAction::setAmount(int32 amount)
{
    if (amount < getMinimum())
        amount = getMinimum();
    if (amount > getMaximum())
        amount = getMaximum();
    if (amount != this->amount) {
        this->amount = amount;
        sig_changed.raise();
    }
}

void
GPlanetSellSuppliesAction::commit()
{
    // commit:
    // - put back supplies
    // - add money
    // FIXME: when we have bought supplies, amount will be negative. we
    // have to do something here.
    // FIXME: addCargo could possibly return failure. what do we do?
    transfer.addCargo(el_Supplies, reserved_sup - amount, true);
    transfer.addCargo(el_Money,    amount, true);
    transfer.commit();

    reserved_sup = amount = 0;

    GTransaction::commit();
}

void
GPlanetSellSuppliesAction::cancel()
{
    // cancel: give back all supplies we have reserved, and don't
    // change money. Actually, ignore this blurb and just cancel the
    // container.
    transfer.cancel();
    amount = reserved_sup = 0;

    GTransaction::cancel();
}
