/**
  *  \file int/core-cmds.cc
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */

#include "int/core-cmds.h"
#include "int/subr.h"
#include "util/console.h"
#include "int/exec.h"
#include "int/data.h"
#include "int/lex.h"

namespace {
    class IntPrintCommand : public IntSubroutine {
     public:
        IntPrintCommand();
        void run(int nargs, string_t name, IntExecutionContext& xc);
    };
}

IntPrintCommand::IntPrintCommand()
{
    setSubrName("PRINT");
}

void
IntPrintCommand::run(int nargs, string_t name, IntExecutionContext& xc)
{
    int arg_index = nargs - 1;

    if (arg_index >= 0 && xc.peekV(arg_index).isFileHandle()) {
        throw IntException("FIXME: unsupported: `print #fd'");
        --arg_index;
    }

    string_t line;
    while (arg_index >= 0) {
        IntObject o = xc.peekV(arg_index);
        if (!o.convertToString())
            throw IntException("Argument to `" + name + "' not convertible to string");
        line.append(o.getStringVal());
        --arg_index;
    }
    console.write(LOG_INFO, line);
    xc.popVs(nargs);
}

void
initCoreCommands()
{
    new IntPrintCommand();
}
