/**
  *  \file ui/window.h
  *  \brief Top-Level Windows and Root Window Declarations
  *
  *  Top-Level windows used for the UI, and a root window to manage
  *  them.
  *
  *  (c) 2001,2002,2003 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UI_WINDOW_H
#define PCC_V2_UI_WINDOW_H

#include "ui/ui.h"
#include "gfx/pixmap.h"
#include "gfx/gfxutil.h"
#include "gfx/multiclip.h"
#include "ui/skin.h"

class UIRoot : public UIWidgetContainer {
    bool stop_requested;
    int  stop_arg;
    GfxMultiClip multi_clip;
 protected:
    Ptr<GfxCanvas> screen;
 public:
    UIRoot(Ptr<GfxCanvas> ascreen);

    virtual void drawBackground(GfxCanvas& can, UpdateInfo& ui);

    virtual GfxCanvas& getCanvasFor(UIWidget* child, ObjectHolder& h);
    void drawBelow(UIWidget* child, GfxCanvas& can, UpdateInfo& ui);
    void draw(GfxCanvas& canvas, UpdateInfo& ui);
    bool handleEvent(const UIEvent& e, bool second_pass);

    int run();
    void stop(int arg);

    void add(UIWidget& w);
    void removeWidget(UIWidget& w);
    void handleChildResize(UIWidget* child, GfxRect oldsize);

    void close(UIWidget& w) { removeWidget(w); }

    void postTimerEvent();

    void centerWidget(UIWidget& w);
    void moveWidgetToEdge(UIWidget& w, int x_pos, int y_pos, int offset = 0);

    void redraw();
    void doDeferredRedraws();
    void sync();
    void getLayoutInfo(LayoutInfo& info);
};

class UIWindow;

class UIWindowSkin : public UISkin {
    UIWindow& window;
 public:
    UIWindowSkin(UIWindow& window)
        : window(window) { }

    const UIColorSet& getColors() const;
    void  outText(GfxCanvas& can,  int f, int x, int y, string_t text);
    void  outTextF(GfxCanvas& can, int f, int x, int y, int lim, string_t text);
    void  drawBackground(GfxCanvas& can, GfxRect r);
};

class UIWindow : public UIGroup {
    static Ptr<GfxPixmap> frame_blue;
    friend class UIWindowSkin;

    UIWindowSkin skin;
 protected:
    string_t name;
    int frame_style, bg_style, border_space;
 public:
    enum {
        BLUE_WINDOW = 0
    };
    UIWindow(const GfxRect& size, string_t aname, int afs, int abs, int aspace = 5);
    UIWindow(string_t aname, int afs, int abs, int aspace = 5);

    bool handleEvent(const UIEvent& e, bool second_pass);

    void draw(GfxCanvas& canvas, UpdateInfo& ui);
    void drawInterior(GfxCanvas& canvas, UpdateInfo& ui);
    GfxRect getClientRectFor(GfxRect ext);
    GfxRect getExtentFor(GfxRect cli);

    static void draw(GfxCanvas& canvas, const GfxRect& extent,
                     int bg_style, string_t name, bool bg=true);
};

#endif
